/*--------------------------------------------------------------------------*/
//	Test program for DirectX9
//
//	by : Ngan Lo, Dec 2002  - all rights reserved
//
//  This test program is same as Tutorial 2 of
//  Direct3D in DirectX9 SDK. Most code are copy from
//  the tutorial and modified to fit in Lccwin32.
/*--------------------------------------------------------------------------*/

#define WIN32_LEAN_AND_MEAN
#define STRICT

// defines for windows
#define WINDOW_CLASS_NAME "NGANLOCLASS"  // WinClass name
#define WINDOWS_TITLE	"Ngan Lo - DirectX9 Test" // WinClass title

#define WINDOW_WIDTH	640
#define WINDOW_HEIGHT	480

// Includes -------------------------------------------------------

#include <time.h>
#include <mmsystem.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <d3d9.h>
#include "dx901.h"

// Prototypes -----------------------------------------------------
#define KEY_DOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)

RECT CalculateWindowRect(HWND hWindow, SIZE szDesiredClient);

// Game logic modules
int Game_Init(void);
int Game_Shutdown(void);
int Game_Main(void);
int Game_Render(void);


// Globals --------------------------------------------------------
LPDIRECT3D9         g_pD3D       = NULL;
LPDIRECT3DDEVICE9   g_pd3dDevice = NULL;
HWND		g_hWnd;
HINSTANCE	g_Hinst;

BOOL	g_bActive;
BOOL	g_bReady;
BOOL	g_bWindowed=TRUE;
BOOL	bD3Dhardware;

LPDIRECT3DVERTEXBUFFER9 g_pVB = NULL;
D3DCOLOR color;

typedef struct { FLOAT x, y, z, rhw; DWORD color; }CUSTOMVERTEX;
#define D3DFVF_CUSTOMVERTEX (D3DFVF_XYZRHW|D3DFVF_DIFFUSE)

//----------------------------------------------------------------------------
// CALLBACK  WindowProc !!!
LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	PAINTSTRUCT	ps;

	switch(msg)
	{
		case WM_CREATE:
       	{
			// do initialization stuff here but NOT DirectX
			return(0);
		} break;
		case WM_ACTIVATE:
		{
			switch((LOWORD(wparam)))
			{
				case WA_INACTIVE:
					g_bActive = FALSE;
					break;
				case WA_CLICKACTIVE:
					g_bActive = TRUE;
					break;
				default:g_bActive = !((BOOL)HIWORD(wparam));break;
			}
		} break;
		case WM_PAINT:
		{
			// start painting
			BeginPaint(hwnd,&ps);
			ValidateRect( hwnd, NULL );

			// end painting
			EndPaint(hwnd,&ps);
			return(0);
		} break;
		case WM_DESTROY:
		{
			// kill the application
			Game_Shutdown();
			PostQuitMessage(0);
			return(0);
		} break;
		case WM_SETCURSOR:
		{
			if (g_bActive && !g_bWindowed)
			{
				SetCursor(NULL);
				return TRUE;
			}
		} break;
		case WM_SYSCOMMAND:
		{
			// Prevent moving/sizing and power loss in fullscreen mode
			switch( wparam )
			{
				case SC_MOVE:
				case SC_SIZE:
				case SC_MAXIMIZE:
				case SC_KEYMENU:
				case SC_MONITORPOWER:
				if( !g_bWindowed )
					return 1;
				break;
			}
		} break;
		case WM_SIZE:
		{
			// Check if we are resizing our window...
			if (SIZE_MAXHIDE==wparam || SIZE_MINIMIZED==wparam)
				g_bActive = FALSE;
			else
				g_bActive = TRUE;
		} break;
		case WM_KEYDOWN:
		{
			switch(wparam)
			{
				// Emergency Exit
				case VK_F12:
					Game_Shutdown();
					PostMessage(hwnd,WM_DESTROY,0,0);
				break;
			}
		} break;
		default:break;
	}
	return (DefWindowProc(hwnd, msg, wparam, lparam));
}

//----------------------------------------------------------------------------
// WinMain
int WINAPI WinMain( HINSTANCE hinstance, HINSTANCE hprevinstance, LPSTR lpcmdline, int ncmdshow )
{
	WNDCLASSEX winclass;
	HWND	 hwnd;
	MSG		 msg;
	RECT rc;

	ZeroMemory(&winclass,sizeof(WNDCLASSEX));
	if (g_bWindowed)
	{
		winclass.cbSize		= sizeof (WNDCLASSEX);
		winclass.style			= CS_HREDRAW | CS_VREDRAW;
		winclass.lpfnWndProc	= WindowProc;
		winclass.cbClsExtra		= 0;
		winclass.cbWndExtra		= 0;
		winclass.hInstance		= hinstance;
		winclass.hIcon			= LoadIcon(hinstance, MAKEINTRESOURCE(IDICON100));
		winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
		winclass.hbrBackground	= GetStockObject(BLACK_BRUSH);
		winclass.lpszMenuName	= NULL;
		winclass.lpszClassName	= WINDOW_CLASS_NAME;
		winclass.hIconSm		= LoadIcon(hinstance, MAKEINTRESOURCE(IDICON100));//(LPCSTR)IDAPPLICON);

		// register the window class
		if (!RegisterClassEx(&winclass))
			return(0);

		// Calculate size of window based on desired client window size
		rc.left = 0;
		rc.top = 0;
		rc.right = WINDOW_WIDTH;
		rc.bottom = WINDOW_HEIGHT;

		hwnd = CreateWindowEx(WS_EX_WINDOWEDGE, WINDOW_CLASS_NAME,
						WINDOWS_TITLE,
						WS_OVERLAPPED|WS_SYSMENU,
						CW_USEDEFAULT, CW_USEDEFAULT,
						rc.right-rc.left, rc.bottom-rc.top,
						NULL, NULL, hinstance, NULL);

		// calculate the new window rect
		SIZE szDesiredClient;

		szDesiredClient.cx = WINDOW_WIDTH;
		szDesiredClient.cy = WINDOW_HEIGHT;

		RECT rcNewWindowRect = CalculateWindowRect(hwnd, szDesiredClient);

		// resize the window according to the new rect
		MoveWindow(hwnd, rcNewWindowRect.left, rcNewWindowRect.top,
			(rcNewWindowRect.right - rcNewWindowRect.left),
			(rcNewWindowRect.bottom - rcNewWindowRect.top), TRUE);
	}
	else
	{
		winclass.cbSize		= sizeof (WNDCLASSEX);
		winclass.style			= CS_DBLCLKS | CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
		winclass.lpfnWndProc	= WindowProc;
		winclass.cbClsExtra		= 0;
		winclass.cbWndExtra		= 0;
		winclass.hInstance		= hinstance;
		winclass.hIcon			= LoadIcon(hinstance, MAKEINTRESOURCE(IDICON100));//IDI_APPLICATION);
		winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
		winclass.hbrBackground	= GetStockObject(BLACK_BRUSH);
		winclass.lpszMenuName	= NULL;
		winclass.lpszClassName	= WINDOW_CLASS_NAME;
		winclass.hIconSm		= LoadIcon(hinstance, MAKEINTRESOURCE(IDICON100));

		// register the window class
		if (!RegisterClassEx(&winclass))
			return(0);

		// Create a full screen window
		hwnd = CreateWindowEx(WS_EX_WINDOWEDGE, WINDOW_CLASS_NAME,
						WINDOWS_TITLE,
						WS_POPUP | WS_VISIBLE,
						0,0,						// x,y
						WINDOW_WIDTH,				// width
						WINDOW_HEIGHT,			// height
						NULL,						// handle to parent
						NULL,						// handle to menu
						hinstance,				// instance
						NULL);				// creation parms
	}

	// Save the global hWnd & hInst
	g_hWnd = hwnd;
	g_Hinst = hinstance;
	SetFocus( g_hWnd );

	ShowWindow(g_hWnd,ncmdshow);
	UpdateWindow(g_hWnd);

	InvalidateRect( g_hWnd, NULL, TRUE );

	Game_Init();

	// Enter main event loop
	while(1)
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{
			if (msg.message == WM_QUIT)
				break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else if (g_bActive)
		{
			Game_Main();
		}
		//else WaitMessage();
	}
	Game_Shutdown();

	return(msg.wParam);
}

//----------------------------------------------------------------------------
//
RECT CalculateWindowRect(HWND hWindow, SIZE szDesiredClient)
{
	// declare a RECT to hold the results of our calculations
	RECT rcDesiredWindowRect;
	memset(&rcDesiredWindowRect, 0, sizeof(rcDesiredWindowRect));

	// get the current window rect and its client rect
	RECT rcCurrentWindowRect;
	RECT rcCurrentClientRect;

	GetWindowRect(hWindow, &rcCurrentWindowRect);
	GetClientRect(hWindow, &rcCurrentClientRect);

	// get the difference between the current and desired client areas
	SIZE szClientDifference;

	szClientDifference.cx = (rcCurrentClientRect.right  - szDesiredClient.cx);
	szClientDifference.cy = (rcCurrentClientRect.bottom - szDesiredClient.cy);

	// get the desired window rect by adding the client area difference to the
	// current window rect
	rcDesiredWindowRect.left   = rcCurrentWindowRect.left;
	rcDesiredWindowRect.top    = rcCurrentWindowRect.top;
	rcDesiredWindowRect.right  = (rcCurrentWindowRect.right  - szClientDifference.cx);
	rcDesiredWindowRect.bottom = (rcCurrentWindowRect.bottom - szClientDifference.cy);

	return rcDesiredWindowRect;
}

void DX9Failed(LPCSTR s, BOOL QUITGAME)
{
	char	szBuff[128];
	va_list vl;

	va_start(vl, s);
	vsprintf(szBuff, s, vl);
	MessageBox(g_hWnd,s,"Error",MB_OK);
	if (QUITGAME)
	{
		PostQuitMessage(0);
	}
}

// Game initialization module here  -----------------------
int Game_Init(void)
{
	srand( (unsigned)time( NULL ) );

	g_pD3D = Direct3DCreate9( D3D_SDK_VERSION );
	if (g_pD3D==NULL)
	{
		DX9Failed("Create D3D failed",TRUE);
		return 0;
	}
	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory( &d3dpp, sizeof(d3dpp) );
	d3dpp.Windowed = TRUE;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
	if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, g_hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &g_pd3dDevice ) ) )
	{
		DX9Failed("CreateDevice failed",TRUE);
		return 0;
	}

	color=D3DCOLOR_XRGB(63,127,127);
	if ( FAILED(g_pd3dDevice->lpVtbl->Clear( g_pd3dDevice, 0, NULL, D3DCLEAR_TARGET, color, 1.0f, 0 )))
	{
		DX9Failed("Clear Screen failed",TRUE);
		return 0;
	}

	// setup polygon vertices, in 2D coordination.
	CUSTOMVERTEX vertices[] =
    {
        { 150.0f,  50.0f, 0.5f, 1.0f, 0xffff0000, },
        { 250.0f, 250.0f, 0.5f, 1.0f, 0xff00ff00, },
        {  50.0f, 250.0f, 0.5f, 1.0f, 0xff0000ff, },
    };
	// Create Vertex buffer
	if( FAILED( g_pd3dDevice->lpVtbl->CreateVertexBuffer( g_pd3dDevice, 3*sizeof(CUSTOMVERTEX), 0, D3DFVF_CUSTOMVERTEX, D3DPOOL_DEFAULT, &g_pVB, NULL ) ) )
	{
		DX9Failed("CreateVertexBuffer failed",TRUE);
		return 0;
	}
	VOID* pVertices;
	if( FAILED( g_pVB->lpVtbl->Lock( g_pVB, 0, sizeof(vertices), (void**)&pVertices, 0 ) ) )
	{
		DX9Failed("Lock VertexBuffer failed",TRUE);
		return 0;
	}
	memcpy( pVertices, vertices, sizeof(vertices) );
	g_pVB->lpVtbl->Unlock(g_pVB);

	return(1);
}

// Game Shutdown module here ------------------------------

int Game_Shutdown(void)
{
	if( g_pVB != NULL)
	{
		g_pVB->lpVtbl->Release(g_pVB);
		g_pVB=NULL;
	}

	if( g_pd3dDevice != NULL)
	{
		g_pd3dDevice->lpVtbl->Release(g_pd3dDevice);
		g_pd3dDevice=NULL;
	}

	if( g_pD3D != NULL)
	{
		g_pD3D->lpVtbl->Release(g_pD3D);
		g_pD3D=NULL;
	}

	return(1);
}

// Game Main-Loop module here -----------------------------
int Game_Render(void)
{
	if (FAILED(g_pd3dDevice->lpVtbl->Clear( g_pd3dDevice, 0, NULL, D3DCLEAR_TARGET, color, 1.0f, 0 )))
	{
		DX9Failed("Clear Screen failed",TRUE);
		return 0;
	}

	if (FAILED(g_pd3dDevice->lpVtbl->BeginScene(g_pd3dDevice)))
	{
		DX9Failed("BeginScene failed",TRUE);
		return 0;
	}

	// Draw the polygon, it's on 2D coordination
	g_pd3dDevice->lpVtbl->SetStreamSource( g_pd3dDevice, 0, g_pVB, 0, sizeof(CUSTOMVERTEX) );
	g_pd3dDevice->lpVtbl->SetFVF( g_pd3dDevice, D3DFVF_CUSTOMVERTEX );
	g_pd3dDevice->lpVtbl->DrawPrimitive( g_pd3dDevice, D3DPT_TRIANGLELIST, 0, 1 );


	if (FAILED(g_pd3dDevice->lpVtbl->EndScene(g_pd3dDevice)))
	{
		DX9Failed("EndScene failed",TRUE);
		return 0;
	}

	return (1);
}

int Game_Main(void)
{

	Game_Render();

	g_pd3dDevice->lpVtbl->Present( g_pd3dDevice, NULL, NULL, NULL, NULL );

	// Exit if ESC key pressed
	if (KEY_DOWN(VK_ESCAPE))
		PostQuitMessage(0);
		//PostMessage(g_hWnd,WM_DESTROY,0,0);

	return(1);
}

